/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <gtk/gtk.h>
#include <glib/gstdio.h>
#include <errno.h>

#include "interface.h"
#include "support.h"
#include "reencode.h"

static gboolean on_fromencodingentry_focus_out_event (
		GtkWidget *widget, GdkEventFocus *event, gpointer user_data);
static void init_filelist (GtkDialog *reencode);
static void init_encoding (GtkDialog *reencode);
static void convert_filelist (GtkDialog *reencode);
static gchar *get_fromencoding (GtkDialog *reencode);
static gchar *get_toencoding (GtkDialog *reencode);
static gboolean modify_encoding (GtkTreeModel *model,
		GtkTreePath *treepath, GtkTreeIter *iter, gpointer data);
static gboolean convert_file (GtkTreeModel *model,
		GtkTreePath *treepath, GtkTreeIter *iter, gpointer data);

void init_reencode ()
{
	GtkDialog *reencode = GTK_DIALOG (create_reencode ());
	GtkWidget *fromencodingentry =
			gtk_bin_get_child (GTK_BIN (
					lookup_widget (GTK_WIDGET (reencode), "fromencoding")));
	g_signal_connect (fromencodingentry, "focus_out_event",
			G_CALLBACK (on_fromencodingentry_focus_out_event), NULL);

	init_filelist (reencode);
	init_encoding (reencode);

	gchar *nautiluspaths = g_strdup (
			g_getenv ("NAUTILUS_SCRIPT_SELECTED_FILE_PATHS"));
	if (nautiluspaths != NULL) {
		gchar **pathv = g_strsplit (nautiluspaths, "\n", 0);
		GSList *paths = NULL;
		gchar **curpath = pathv;
		while (*curpath) {
			if (*curpath[0] != '\0')
				paths = g_slist_prepend (paths, g_strdup (*curpath));
			curpath++;
		}
		add_filename (reencode, paths);
		g_strfreev (pathv);
		g_free (nautiluspaths);
	}

	if (gtk_dialog_run (reencode) == GTK_RESPONSE_OK)
		convert_filelist (reencode);

	gtk_widget_destroy (GTK_WIDGET (reencode));
}

enum { PATH_ACTIVE, PATH_BASE, PATH_ORIG, PATH_DIR, PATH_FULL, NUM_PATH };

void add_filename (GtkDialog *reencode, GSList *paths)
{
	GtkTreeView *filelist = GTK_TREE_VIEW (
			lookup_widget (GTK_WIDGET (reencode), "filelist"));

	gchar *fromenc = get_fromencoding (reencode);

	GtkListStore *store = GTK_LIST_STORE (gtk_tree_view_get_model (filelist));
	GtkTreeIter iter;

	GSList *current = paths;
	while (current) {
		gtk_list_store_append (store, &iter);
		const gchar *path = current->data;

		gchar *dir = g_path_get_dirname (path);
		gchar *base = g_path_get_basename (path);
		gchar *orig = g_convert_with_fallback (
				base, -1, "UTF-8", fromenc,	base, NULL, NULL, NULL);
		if (orig == NULL)
			orig = g_strdup (base);
	
		gchar *dirdisp = g_filename_display_name (dir);
		gchar *basedisp = g_filename_display_name (base);
		gchar *origdisp = g_filename_display_name (orig);

		gtk_list_store_set (store, &iter, PATH_ACTIVE, TRUE,
				PATH_BASE, basedisp, PATH_ORIG, origdisp, PATH_DIR, dirdisp,
				PATH_FULL, path, -1);

		g_free (dir);
		g_free (base);
		g_free (orig);
		g_free (basedisp);
		g_free (origdisp);
		g_free (dirdisp);

		g_free (current->data);
		current = g_slist_next (current);
	}
	g_free (fromenc);
	g_slist_free (paths);
}

void update_filename (GtkDialog *reencode)
{
	GtkTreeView *filelist = GTK_TREE_VIEW (
			lookup_widget (GTK_WIDGET (reencode), "filelist"));
	gchar *fromenc = get_fromencoding (reencode);
	gtk_tree_model_foreach (
			gtk_tree_view_get_model (filelist), modify_encoding, fromenc);
	g_free (fromenc);
}

static gboolean on_fromencodingentry_focus_out_event (
		GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
	GtkDialog *reencode = GTK_DIALOG (lookup_widget (widget, "reencode"));
	update_filename (reencode);
	return FALSE;
}

static void init_encoding (GtkDialog *reencode)
{
	GtkComboBoxEntry *fromencoding = GTK_COMBO_BOX_ENTRY (
			lookup_widget (GTK_WIDGET (reencode), "fromencoding"));
	GtkComboBoxEntry *toencoding = GTK_COMBO_BOX_ENTRY (
			lookup_widget (GTK_WIDGET (reencode), "toencoding"));
	
	const gchar **charsets;
	g_get_filename_charsets (&charsets);
	
	GtkEntry *fromencodingentry =
		GTK_ENTRY (gtk_bin_get_child (GTK_BIN (fromencoding)));
	gtk_entry_set_text (fromencodingentry, *charsets);
	GtkEntry *toencodingentry =
		GTK_ENTRY (gtk_bin_get_child (GTK_BIN (toencoding)));
	gtk_entry_set_text (toencodingentry, *charsets);

	while (*charsets) {
		gtk_combo_box_append_text (GTK_COMBO_BOX (fromencoding), *charsets);
		gtk_combo_box_append_text (GTK_COMBO_BOX (toencoding), *charsets);
		charsets++;
	}
}

static void toggle_filelist (GtkCellRendererToggle *cell_renderer,
		gchar *path_string, gpointer user_data)
{
	GtkListStore *store = user_data;
	GtkTreeIter iter;
	GtkTreePath *treepath = gtk_tree_path_new_from_string (path_string);
	gboolean active;
	
	gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, treepath);
	gtk_tree_model_get (GTK_TREE_MODEL (store),
			&iter, PATH_ACTIVE, &active, -1);
	active = !active;
	gtk_list_store_set (store, &iter, PATH_ACTIVE, active, -1);
	gtk_tree_path_free (treepath);
}

static void init_filelist (GtkDialog *reencode)
{
	GtkTreeView *filelist = GTK_TREE_VIEW (
			lookup_widget (GTK_WIDGET (reencode), "filelist"));
	
	GtkListStore *store = gtk_list_store_new (NUM_PATH, G_TYPE_BOOLEAN,
			G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	gtk_tree_view_set_model (filelist, GTK_TREE_MODEL (store));
	g_object_unref (store);
	
	GtkCellRenderer *toggle = gtk_cell_renderer_toggle_new ();
	g_signal_connect ((gpointer) toggle, "toggled",
			G_CALLBACK (toggle_filelist), store);
	gtk_tree_view_insert_column_with_attributes (filelist, -1,
			"",	toggle,	"active", PATH_ACTIVE, NULL);
	gtk_tree_view_insert_column_with_attributes (filelist, -1,
			_("Broken filename"), gtk_cell_renderer_text_new (),
			"text", PATH_BASE, NULL);
	gtk_tree_view_insert_column_with_attributes (filelist, -1,
			_("Original filename"), gtk_cell_renderer_text_new (),
			"text", PATH_ORIG, NULL);
	gtk_tree_view_insert_column_with_attributes (filelist, -1,
			_("Directory"), gtk_cell_renderer_text_new (),
			"text", PATH_DIR, NULL);
}

static void convert_filelist (GtkDialog *reencode){
	GtkTreeView *filelist = GTK_TREE_VIEW (
			lookup_widget (GTK_WIDGET (reencode), "filelist"));
	gchar *enc[2] = {
		get_fromencoding (reencode),
		get_toencoding (reencode)
	};
	gtk_tree_model_foreach (
			gtk_tree_view_get_model (filelist), convert_file, enc);
	g_free (enc[0]);
	g_free (enc[1]);
}

static gchar *get_fromencoding (GtkDialog *reencode)
{
	GtkComboBoxEntry *fromencoding = GTK_COMBO_BOX_ENTRY (
			lookup_widget (GTK_WIDGET (reencode), "fromencoding"));
	return gtk_combo_box_get_active_text (GTK_COMBO_BOX (fromencoding));
}

static gchar *get_toencoding (GtkDialog *reencode)
{
	GtkComboBoxEntry *toencoding = GTK_COMBO_BOX_ENTRY (
			lookup_widget (GTK_WIDGET (reencode), "toencoding"));
	return gtk_combo_box_get_active_text (GTK_COMBO_BOX (toencoding));
}

static gboolean modify_encoding (GtkTreeModel *model,
		GtkTreePath *treepath, GtkTreeIter *iter, gpointer data)
{
	GValue value = { 0, };
	gtk_tree_model_get_value (model, iter, PATH_FULL, &value);

	gchar *fromenc = data;
	gchar *base = g_path_get_basename (g_value_get_string (&value));
	gchar *orig = g_convert_with_fallback (
			base, -1, "UTF-8", fromenc,	base, NULL, NULL, NULL);
	if (orig == NULL)
		orig = g_strdup (base);

	gchar *origdisp = g_filename_display_name (orig);

	gtk_list_store_set (GTK_LIST_STORE (model), iter, PATH_ORIG, origdisp, -1);

	g_value_unset (&value);
	g_free (base);
	g_free (orig);
	g_free (origdisp);
	return FALSE;
}

static gboolean convert_file (GtkTreeModel *model,
		GtkTreePath *treepath, GtkTreeIter *iter, gpointer data)
{
	GValue val_active = { 0, };
	gtk_tree_model_get_value (model, iter, PATH_ACTIVE, &val_active);
	if (g_value_get_boolean (&val_active) == FALSE) {
		g_value_unset (&val_active);
		return TRUE;
	}
	
	GValue value = { 0, };
	gtk_tree_model_get_value (model, iter, PATH_FULL, &value);

	gchar **enc = data;
	const gchar *path = g_value_get_string (&value);
	gchar *dir = g_path_get_dirname (path);
	gchar *base = g_path_get_basename (path);
	gchar *conv = g_convert_with_fallback (
			base, -1, enc[1], enc[0], base, NULL, NULL, NULL);
	if (conv == NULL)
		conv = g_strdup (base);

	gchar *newpath = g_build_filename (dir, conv, NULL);
	if (g_rename (path, newpath) == -1) {
		gchar *pathdisp = g_filename_display_name (path);
		gchar *newpathdisp = g_filename_display_name (newpath);
		gchar *errmsg = g_strdup_printf (
				_("Error occured while moving\nfrom %s\nto %s:\n%s."),
				pathdisp, newpathdisp, g_strerror (errno));
		GtkDialog *error = GTK_DIALOG (create_error ());
		GtkLabel *content = GTK_LABEL (
				lookup_widget (GTK_WIDGET (error), "content"));
		gtk_label_set_text (content, errmsg);
		gtk_dialog_run (error);
		gtk_widget_destroy (GTK_WIDGET (error));
		g_free (pathdisp);
		g_free (newpathdisp);
		g_free (errmsg);
	}

	g_value_unset (&val_active);
	g_value_unset (&value);
	g_free (dir);
	g_free (base);
	g_free (conv);
	g_free (newpath);
	return FALSE;
}
